/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <ui-gadget.h>
#include <app_debug.h>
#include <viewmgr.h>
#include <inputmgr.h>
#include <app_define.h>

#include "defs.h"
#include "view.h"
#include "datamgr.h"

enum pin_view_type {
	VIEW_CHECK_PIN,
	VIEW_SWITCH_PIN
};

struct _priv {
	Evas_Object *win;

	ui_gadget_h ug;

	char *pincode;
	enum pin_view_type type;

	struct datamgr *dm;
	struct datamgr_item *di;
};

static void _send_message(struct _priv *priv, const char *result)
{
	app_control_h app_control;
	int r;

	r = app_control_create(&app_control);
	if (r != APP_CONTROL_ERROR_NONE) {
		_ERR("failed to create app_control");
		return;
	}

	r = app_control_add_extra_data(app_control, PARAM_PIN, result);
	if (r != APP_CONTROL_ERROR_NONE) {
		_ERR("failed to add data");
		app_control_destroy(app_control);
		return;
	}

	ug_send_message(priv->ug, app_control);

	app_control_destroy(app_control);
}

static void _check_pincode(struct _priv *priv, char *pincode)
{
	switch (priv->type) {
	case VIEW_CHECK_PIN:
		if (!strcmp(pincode, priv->pincode)) {
			viewmgr_hide_view(VIEW_PIN);
			viewmgr_update_view(VIEW_USER_EDIT,
					UPDATE_TYPE_PIN_RESULT, pincode);
		} else {
			_send_message(priv, MESSAGE_FAIL);
		}
		break;
	case VIEW_SWITCH_PIN:
		priv->di->parameter = pincode;
		if (datamgr_select_item(priv->dm, priv->di) !=
				ITEM_SELECT_ERROR_NONE)
			_send_message(priv, MESSAGE_FAIL);
		break;
	default:
		break;
	}
}

static void _ug_result_cb(ui_gadget_h ug, app_control_h result, void *p)
{
	char *pincode;
	int r;

	if (!p) {
		_ERR("Invalid argument");
		return;
	}

	r = app_control_get_extra_data(result, PARAM_PIN, &pincode);
	if (r != APP_CONTROL_ERROR_NONE || !pincode) {
		_ERR("failed to get data");
		return;
	}

	_check_pincode(p, pincode);

	free(pincode);
}

static void _ug_destroy_cb(ui_gadget_h ug, void *p)
{
	viewmgr_hide_view(VIEW_PIN);
}

static ui_gadget_h _load_ug(struct _priv *priv)
{
	ui_gadget_h ug;
	struct ug_cbs cb = {0,};

	cb.result_cb = _ug_result_cb;
	cb.destroy_cb = _ug_destroy_cb;
	cb.priv = priv;

	ug = ug_create(NULL, UG_ID_PIN, UG_MODE_FRAMEVIEW, NULL, &cb);

	return ug;
}

static Evas_Object *_create(Evas_Object *win, void *data)
{
	struct _priv *priv;

	if (!win || !data) {
		_ERR("Invalid argument");
		return NULL;
	}

	priv = calloc(1, sizeof(*priv));
	if (!priv) {
		_ERR("failed to calloc priv");
		return NULL;
	}

	priv->win = win;
	priv->dm = data;

	viewmgr_set_view_data(VIEW_PIN, priv);

	return win;
}

static void _show(void *data)
{
	struct _priv *priv;
	ui_gadget_h ug;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	priv = data;

	ug = _load_ug(priv);
	if (!ug) {
		_ERR("failed to create ug");
		return;
	}

	priv->ug = ug;
}

static void _hide(void *data)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	priv = data;

	if (priv->ug)
		ug_destroy(priv->ug);

	free(priv->pincode);
	priv->pincode = NULL;
	priv->di = NULL;
}

static void _destroy(void *data)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	priv = data;

	free(priv);
}

static void _update(void *view_data, int update_type, void *data)
{
	struct _priv *priv;

	if (!view_data || !data) {
		_ERR("Invalid argument");
		return;
	}

	priv = view_data;

	switch (update_type) {
	case UPDATE_TYPE_PIN_CODE:
		priv->pincode = strdup(data);
		priv->type = VIEW_CHECK_PIN;
		break;
	case UPDATE_TYPE_SWITCH:
		priv->di = data;
		priv->type = VIEW_SWITCH_PIN;
		break;
	default:
		_ERR("Invalid type");
		return;
	}
}

static view_class vclass = {
	.view_id = VIEW_PIN,
	.create = _create,
	.show = _show,
	.hide = _hide,
	.destroy = _destroy,
	.update = _update
};

view_class *view_pin_get_vclass(void)
{
	return &vclass;
}
